/*
 * Copyright 2009 Red Hat, Inc.
 * Red Hat licenses this file to you under the Apache License, version
 * 2.0 (the "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *    http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.  See the License for the specific language governing
 * permissions and limitations under the License.
 */

package org.hornetq.jms.example;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.Session;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.naming.InitialContext;

import org.hornetq.common.example.HornetQExample;

/**
 * An example with QueueBrowser and a MessageListener to illustrate odd behaivior
 */
public class QueueBrowserListenerExample extends HornetQExample
{
   public static void main(final String[] args)
   {
      new QueueBrowserListenerExample().run(args);
   }

   @Override
   public boolean runExample() throws Exception
   {
      Connection connection = null;
      InitialContext initialContext = null;
      try
      {
	     // Normal setup
         initialContext = getContext(0);
         Queue queue = (Queue)initialContext.lookup("/queue/exampleQueue");
         ConnectionFactory cf = (ConnectionFactory)initialContext.lookup("/ConnectionFactory");
         connection = cf.createConnection();
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         MessageProducer producer = session.createProducer(queue);

		 // Send several messages
         TextMessage message_1 = session.createTextMessage("this is the 1st message.");
         TextMessage message_2 = session.createTextMessage("this is the 2nd message.");
         TextMessage message_3 = session.createTextMessage("this is the 3rd message.");
         TextMessage message_4 = session.createTextMessage("this is the 4th message.");
         TextMessage message_5 = session.createTextMessage("this is the 5th message.");
         TextMessage message_6 = session.createTextMessage("this is the 6th message.");
         TextMessage message_7 = session.createTextMessage("this is the 7th message.");
         TextMessage message_8 = session.createTextMessage("this is the 8th message.");
         TextMessage message_9 = session.createTextMessage("this is the 9th message.");
         TextMessage message_10 = session.createTextMessage("this is the 10th message");

         producer.send(message_1);
         producer.send(message_2);
         producer.send(message_3);
         producer.send(message_4);
         producer.send(message_5);
         producer.send(message_6);
         producer.send(message_7);
         producer.send(message_8);
         producer.send(message_9);
         producer.send(message_10);

		 // Set up a consumer with a listener
         MessageConsumer messageConsumer = session.createConsumer(queue);
         messageConsumer.setMessageListener(new MessageListener()
         {
            public void onMessage(final Message message)
            {
               TextMessage textMessage = (TextMessage)message;
			   System.out.println("------------------------");
               try
               {
                  System.out.println("Received notification: " + textMessage.getText());
				  // Simulate processing time
                  Thread.sleep(5000);
               }
               catch (Exception e)
               {
               }
               System.out.println("------------------------");
            }
         });

         connection.start();

		 // This browses 10 times per message processed until the 10th message is done processing + one more browse
         for (int i = 0; i < 101; ++i)
		 {
		    /* 
			* I would expect the output here to be all messages from the first non-buffered to the 10th message.
			* It is not.  The messages browsed are always consecutive and always begin with the next non-
			* buffered message, but the number of messages listed is not consistent.  Why does the list not
			* always end with the 10th message?
			*
			* Increasing the value of consumer-window-size in hornetq-jms.xml increases both the number of
			* messages buffered and the possible number of messages browsed.
			* 
			* Further testing shows each of these messages will fit in a buffer of 300 bytes so using
			* multiples of 300 for consumer-window-size is informative.  It seems that the number of
			* messages browsed is always AT LEAST as many messages as fit in the buffer.
			* consumer-window-size = 300: 1 message buffered, AT LEAST 1 message browsed (until all are buffered)
			* consumer-window-size = 600: 2 messages buffered, AT LEAST 2 messages browsed
			* consumer-window-size = 900: 3 messages buffered, AT LEAST 3 messages browsed
			* consumer-window-size = 1200: 4 messages buffered, AT LEAST 4 messages browsed
			* consumer-window-size = 1500: 5 messages buffered, 5 messages browsed (there are only 10)
			*
			* Is there another configuration option I can use to control the number of messages browsed or
			* can I somehow divorce consumer-window-size from the number of messages browsed?
			*/
            browse(session, queue);
		 }
		 
         return true;

      }
      finally
      {
	     // Clean up
         if (initialContext != null)
         {
            initialContext.close();
         }
         if (connection != null)
         {
            connection.close();
         }
      }
   }
   
   private void browse(Session session, Queue queue) throws JMSException, InterruptedException {
         QueueBrowser browser = session.createBrowser(queue);
         Enumeration messageEnum = browser.getEnumeration();
		 
         System.out.println("Begin Browsing");
         while (messageEnum.hasMoreElements())
         {
            TextMessage message = (TextMessage)messageEnum.nextElement();
            System.out.println("Browsing: " + message.getText());
         }
         System.out.println("End Browsing");
		 Thread.sleep(500);

         browser.close();
   }
}
